@@ -30,6 +30,14 @@ grid.push_row(vec![7,8,9]);
 assert_eq!(grid, grid![[1,2,3][4,5,6][7,8,9]])
  ```
 */
+
+#![cfg_attr(not(feature = "std"), no_std)]
+
+#[cfg(not(feature = "std"))]
+extern crate no_std_compat as std;
+#[cfg(not(feature = "std"))]
+use std::prelude::v1::*;
+
 use std::cmp::Eq;
 use std::fmt;
 use std::iter::StepBy;
@@ -133,8 +141,8 @@ impl<T: Clone> Grid<T> {
         }
         Grid {
             data: vec![T::default(); rows * cols],
-            cols: cols,
-            rows: rows,
+            cols,
+            rows,
         }
     }
 
@@ -145,8 +153,8 @@ impl<T: Clone> Grid<T> {
         }
         Grid {
             data: vec![data; rows * cols],
-            cols: cols,
-            rows: rows,
+            cols,
+            rows,
         }
     }
 
@@ -162,8 +170,8 @@ impl<T: Clone> Grid<T> {
     /// ```
     ///
     /// will create a grid with the following layout:
-    /// [1,2,3]
-    /// [4,5,6]
+    /// \[1,2,3\]
+    /// \[4,5,6\]
     ///
     /// This example will fail, because `vec.len()` is not a multiple of `cols`:
     ///
@@ -175,11 +183,11 @@ impl<T: Clone> Grid<T> {
         let rows = vec.len();
         if rows == 0 {
             if cols == 0 {
-                return Grid {
+                Grid {
                     data: vec![],
                     rows: 0,
                     cols: 0,
-                };
+                }
             } else {
                 panic!("Vector length is zero, but cols is {:?}", cols);
             }
@@ -189,13 +197,16 @@ impl<T: Clone> Grid<T> {
             Grid {
                 data: vec,
                 rows: rows / cols,
-                cols: cols,
+                cols,
             }
         }
     }
 
     /// Returns a reference to an element, without performing bound checks.
     /// Generally not recommended, use with caution!
+    ///
+    /// # Safety
+    ///
     /// Calling this method with an out-of-bounds index is undefined behavior even if the resulting reference is not used.
     #[inline]
     pub unsafe fn get_unchecked(&self, row: usize, col: usize) -> &T {
@@ -204,6 +215,9 @@ impl<T: Clone> Grid<T> {
 
     /// Returns a mutable reference to an element, without performing bound checks.
     /// Generally not recommended, use with caution!
+    ///
+    /// # Safety
+    /// 
     /// Calling this method with an out-of-bounds index is undefined behavior even if the resulting reference is not used.
     #[inline]
     pub unsafe fn get_unchecked_mut(&mut self, row: usize, col: usize) -> &mut T {
@@ -369,7 +383,7 @@ impl<T: Clone> Grid<T> {
     pub fn iter_row(&self, row: usize) -> Iter<T> {
         if row < self.rows {
             let start = row * self.cols;
-            return self.data[start..(start + self.cols)].iter();
+            self.data[start..(start + self.cols)].iter()
         } else {
             panic!(
                 "out of bounds. Row must be less than {:?}, but is {:?}.",
@@ -398,7 +412,7 @@ impl<T: Clone> Grid<T> {
         if row < self.rows {
             let cols = self.cols;
             let start = row * cols;
-            return self.data[start..(start + cols)].iter_mut();
+            self.data[start..(start + cols)].iter_mut()
         } else {
             panic!(
                 "out of bounds. Row must be less than {:?}, but is {:?}.",
@@ -514,9 +528,9 @@ impl<T: Clone> Grid<T> {
             if self.rows == 0 {
                 self.cols = 0;
             }
-            return Some(row);
+            Some(row)
         } else {
-            return None;
+            None
         }
     }
 
@@ -539,16 +553,15 @@ impl<T: Clone> Grid<T> {
             let mut col = Vec::with_capacity(self.rows);
             for i in 0..self.rows {
                 let idx = i * self.cols + self.cols - 1 - i;
-                println!["{:?}", idx];
                 col.push(self.data.remove(idx));
             }
             self.cols -= 1;
             if self.cols == 0 {
                 self.rows = 0;
             }
-            return Some(col);
+            Some(col)
         } else {
-            return None;
+            None
         }
     }
 
@@ -624,6 +637,21 @@ impl<T: Clone> Grid<T> {
     pub fn into_vec(self) -> Vec<T> {
         self.data
     }
+
+    /// Transpose the grid so that columns become rows in new grid.
+    pub fn transpose(&self) -> Grid<T> {
+        let mut data = Vec::with_capacity(self.data.len());
+        for c in 0..self.cols {
+            for r in 0..self.rows {
+                data.push(self[r][c].clone());
+            }
+        }
+        Grid {
+            data,
+            cols: self.rows,
+            rows: self.cols,
+        }
+    }
 }
 
 impl<T: Clone> Clone for Grid<T> {
@@ -654,7 +682,7 @@ impl<T: Clone> Index<usize> for Grid<T> {
 
 impl<T: Clone> IndexMut<usize> for Grid<T> {
     fn index_mut(&mut self, idx: usize) -> &mut Self::Output {
-        &mut self.data[(idx * &self.cols)..]
+        &mut self.data[(idx * self.cols)..]
     }
 }
 
@@ -810,11 +838,11 @@ mod test {
         grid.push_col(vec![3, 13]);
         assert_eq!(grid.size(), (2, 4));
         assert_eq!(
-            grid.iter_row(0).map(|x| *x).collect::<Vec<_>>(),
+            grid.iter_row(0).copied().collect::<Vec<_>>(),
             vec![0, 1, 2, 3]
         );
         assert_eq!(
-            grid.iter_row(1).map(|x| *x).collect::<Vec<_>>(),
+            grid.iter_row(1).copied().collect::<Vec<_>>(),
             vec![10, 11, 12, 13]
         );
     }
@@ -828,15 +856,15 @@ mod test {
         grid.push_col(vec!['x', 'y', 'z']);
         assert_eq!(grid.size(), (3, 5));
         assert_eq!(
-            grid.iter_row(0).map(|x| *x).collect::<Vec<_>>(),
+            grid.iter_row(0).copied().collect::<Vec<_>>(),
             vec!['a', 'b', 'c', 'd', 'x']
         );
         assert_eq!(
-            grid.iter_row(1).map(|x| *x).collect::<Vec<_>>(),
+            grid.iter_row(1).copied().collect::<Vec<_>>(),
             vec!['a', 'b', 'c', 'd', 'y']
         );
         assert_eq!(
-            grid.iter_row(2).map(|x| *x).collect::<Vec<_>>(),
+            grid.iter_row(2).copied().collect::<Vec<_>>(),
             vec!['a', 'b', 'c', 'd', 'z']
         );
     }
@@ -1126,4 +1154,10 @@ mod test {
         let grid = Grid::init(1, 2, 3);
         assert_eq!(grid.size(), (1, 2));
     }
+
+    #[test]
+    fn transpose() {
+        let grid: Grid<u8> = grid![[1,2,3][4,5,6]];
+        assert_eq!(format!("{:?}", grid.transpose()), "[[1, 4][2, 5][3, 6]]");
+    }
 }
